import numpy as np
import pandas as pd

# scaling methods
from .bipartite_scaling.factor_analysis import *


class DiBipartite():

	"""
	Directed bipartite graph, stored as a list of edges in a pandas DataFrame.
	Nodes are identified with a string label.
	Edge multiplicity is 1 (no repeated edges).
	Bipartite has two sets of nodes: top, bottom.
	All edges go from bottom to top.

	Storage convention in the CSV file is,
	for each row/edge:

	bottom_node_i,top_node_j

	"""

	edges_df = []
	top_nodes_list = []
	bottom_nodes_list = []
	nodes_list = []
	edges = np.nan
	top_nodes = np.nan
	bottom_nodes = np.nan

	info = []
	embedding = []

	def __init__(self,filename,header=None,order=['bottom','top']):

		# Loading the list of edges
		df = pd.read_csv(filename,header=header,names=order,dtype={'bottom':'str','top':'str'})

		# Checks
		if df.shape[0] == 0:
			raise ValueError('No edges in file.')
		if df.shape[1] > 2:
			raise ValueError('File has more than two columns. Format is csv of N_edges x 2.')
		if df.duplicated().sum()>0:
			print('Deleting were %d duplicated edges...'%(df.duplicated().sum()))
			df=df[~df.duplicated()]
		if df.isna().any().any():
			print('Deleting %d edges with NaN values...'%((df['bottom'].isna()|df['top'].isna()).sum()))
			df.dropna(axis=0,how='any',inplace=True)
		if np.intersect1d(df['bottom'],df['top']).size>0:
			raise ValueError('Top and bottom nodes are not disjoint sets.')
		if df.shape[0] == 0:
			raise ValueError('After deleting repeated and NaN edges, there were none left.')

		# storing values
		self.edges_df = df.copy(deep=True)
		del df
		self.top_nodes_list = self.edges_df['top'].unique().tolist()
		self.bottom_nodes_list = self.edges_df['bottom'].unique().tolist()
		self.nodes_list = self.top_nodes_list + self.bottom_nodes_list
		self.edges = self.edges_df.shape[0]
		self.top_nodes = len(self.top_nodes_list)
		self.bottom_nodes = len(self.bottom_nodes_list)

		return

	# Scaling methods
	def CA(self,theta=3,dimensions=3,coordinates='top',all_nodes=False):
		embedding,info = CorrespondanceAnalysis(self.edges_df,theta=theta,
												dimensions=dimensions,coordinates=coordinates,
												all_nodes=all_nodes)
		self.info = info
		self.embedding = embedding
		return;








import numpy as np
import pandas as pd
import prince

def CorrespondanceAnalysis(df,theta=1,dimensions=3,coordinates='top',all_nodes=False):

	# Distinguishing coordinate edges from entities nodes
	entities = 'bottom' if coordinates=='top' else 'top'

	# Degree of entity nodes
	degrees = df.groupby(entities).count()[coordinates].sort_values(ascending=False)

	# filtering edge list to delete nodes that 
	df = df[df[entities].isin(degrees[degrees>=theta].index.values)]

	df['w']=1

	# Prototype version: Pivoting adjacency matrix in chunks
	# chunk_size = 100000
	# chunks = [x for x in range(0, df.shape[0], chunk_size)]
	# M = pd.concat([df.loc[df[entities].isin(degrees[degrees>=theta].index.values),[entities,coordinates,'w']].iloc[ chunks[i]:chunks[i + 1] - 1 ].pivot(index=entities, columns=coordinates, values='w') for i in range(0, len(chunks) - 1)])

	# Current version: pivoting adjacency matrix
	# Will fail for large sizes (long tailed degree distributions, theta=1)
	M = df[[entities,coordinates,'w']].pivot(index=entities,columns=coordinates,values='w')
	M.fillna(0,inplace=True)      

	# Selecting the core sub-graph that will be used in the CA
	M_mask = (~M.duplicated())
	selected_rows = M.loc[M_mask].index.values
	unselected_rows = M.loc[~M_mask].index.values

	# Correspondent Analysis
	ca = prince.CA(n_components=dimensions,
				   n_iter=4,copy=True,check_input=True,
				   engine='auto',random_state=np.random.randint(1,100))
	ca = ca.fit(M[M_mask])
	if all_nodes:
	    row_coords = ca.row_coordinates(M)
	else:
	    row_coords = ca.row_coordinates(M[M_mask])
	col_coords = ca.column_coordinates(M[M_mask])

	output=pd.concat([row_coords,col_coords],axis=0)

	info = {'explained_inertia':ca.explained_inertia_,}

	return output,info;



"""`setup.py`"""
from setuptools import setup, find_packages

# # Package requirements
# with open('requirements.txt') as f:
#     INSTALL_REQUIRES = [l.strip() for l in f.readlines() if l]

INSTALL_REQUIRES=[
'numpy',
'pandas',
'scikit-learn',
'scipy',
'matplotlib',
'prince',
]

setup(name='mdscaling',
      version='0.0.7',
      description='Multi-dimensional network scaling',
      author='Pedro Ramaciotti Morales',
      author_email='pedro.ramaciotti@gmail.com',
      url = 'https://github.com/pedroramaciotti/MDScaling',
      download_url = 'https://github.com/pedroramaciotti/MDScaling/archive/0.0.7.tar.gz',
      keywords = ['multi-dimensional scaling','graph compression','dimentionality reduction','representation learning'],
      packages=find_packages(),
      data_files=[('', ['LICENSE'])],
      install_requires=INSTALL_REQUIRES)
